/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.application;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.FlavorEvent;
import java.awt.datatransfer.FlavorListener;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id:ClipboardService.java 220 2006-03-06 14:46:23Z bombe $
 */
public class ClipboardService extends AbstractService implements FlavorListener {

	public static final String NAME = "Clipboard Service";
	private Set<ClipboardListener> clipboardListeners;
	private Clipboard systemClipboard;
	private String lastClipboardContent;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getName() {
		return NAME;
	}

	public void addClipboardListener(ClipboardListener clipboardListener) {
		clipboardListeners.add(clipboardListener);
	}

	public void removeClipboardListener(ClipboardListener clipboardListener) {
		clipboardListeners.remove(clipboardListener);
	}

	private void fireUrlFound(String url) {
		for (ClipboardListener clipboardListener: clipboardListeners) {
			clipboardListener.urlFound(url);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init() {
		super.init();
		systemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
		clipboardListeners = new HashSet<ClipboardListener>();
		lastClipboardContent = null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void destroy() {
		super.destroy();
		lastClipboardContent = null;
		systemClipboard = null;
		clipboardListeners.clear();
		clipboardListeners = null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void start() {
		systemClipboard.addFlavorListener(this);
		setWaitPeriod(0);
		super.start();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void stop() {
		systemClipboard.removeFlavorListener(this);
		super.stop();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void work() {
		String clipboardContent = getClipboardContent();
		if (clipboardContent != null) {
			if (!clipboardContent.equals(lastClipboardContent)) {
				StringTokenizer urlTokenizer = new StringTokenizer(clipboardContent, " \t\n");
				while (urlTokenizer.hasMoreTokens()) {
					String url = urlTokenizer.nextToken();
					if (url.startsWith("freenet:CHK@") || url.startsWith("CHK@")) {
						fireUrlFound(url.startsWith("freenet:") ? url.substring(8) : url);
					}
				}
				lastClipboardContent = clipboardContent;
			}
		}
	}

	private String getClipboardContent() {
		Transferable content = systemClipboard.getContents(this);
		if (content.isDataFlavorSupported(DataFlavor.stringFlavor)) {
			String stringContent;
			try {
				stringContent = (String) content.getTransferData(DataFlavor.stringFlavor);
				return stringContent;
			} catch (UnsupportedFlavorException e) {
			} catch (IOException e) {
			}
		}
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public void flavorsChanged(FlavorEvent e) {
		synchronized (syncObject) {
			syncObject.notify();
		}
	}

}
