/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.application;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import de.todesbaum.freemule.main.Version;
import de.todesbaum.util.freenet.fcp.FcpException;
import de.todesbaum.util.freenet.fcp.FcpNode;


/**
 * @author <a href="mailto:droden@gmail.com">David Roden </a>
 * @version $Id:FreeMule.java 220 2006-03-06 14:46:23Z bombe $
 */
public class FreeMule extends AbstractService implements ServiceListener, AnnouncementListener, DiscoveryListener, DownloadListener, FilelistCollectorListener, FilelistInsertListener, SharedFileListener, UploadRequestDiscoveryListener, UploadRequestInsertListener, ClientListener, ClipboardListener, UploadListener {

	public static final String NAME = "freeMule";

	private List<ConfigurationListener> configurationListeners;
	private Client thisClient;
	private String privateKey;
	private String publicKey;
	private String entropy;
	private String username;
	private String nodeAddress;
	private int nodePort;
	private String networkName;
	private int hopsToLive;
	private String incomingDirectory;
	private String tempDirectory;
	private SharedDirectory[] sharedDirectories;
	private int filelistIndex;
	private int filelistLastInsertedVersion;

	private AnnouncementService announcementService;
	private ClientService clientService;
	private ClipboardService clipboardService;
	private DiscoveryService discoveryService;
	private DownloadService downloadService;
	private EngineService engineService;
	private FilelistService filelistService;
	private FilelistInsertService filelistInsertService;
	private FilelistCollectorService filelistCollectorService;
	private SharedFileService sharedFileService;
	private UploadRequestInsertService uploadRequestInsertService;
	private UploadRequestDiscoveryService uploadRequestDiscoveryService;
	private UploadService uploadService;

	public String getName() {
		return NAME;
	}

	public void addConfigurationListener(ConfigurationListener configurationListener) {
		configurationListeners.add(configurationListener);
	}

	public void removeConfigurationListener(ConfigurationListener configurationListener) {
		configurationListeners.remove(configurationListener);
	}

	private void fireConfigurationChanged() {
		for (ConfigurationListener configurationListener: configurationListeners) {
			configurationListener.configurationChanged(this);
		}
	}

	/**
	 * @return Returns the privateKey.
	 */
	public String getPrivateKey() {
		return privateKey;
	}

	/**
	 * @param privateKey
	 *            The privateKey to set.
	 */
	public void setPrivateKey(String privateKey) {
		this.privateKey = privateKey;
	}

	/**
	 * @return Returns the publicKey.
	 */
	public String getPublicKey() {
		return publicKey;
	}

	/**
	 * @param publicKey
	 *            The publicKey to set.
	 */
	public void setPublicKey(String publicKey) {
		this.publicKey = publicKey;
	}

	/**
	 * @return Returns the entropy.
	 */
	public String getEntropy() {
		return entropy;
	}

	/**
	 * @param entropy
	 *            The entropy to set.
	 */
	public void setEntropy(String entropy) {
		this.entropy = entropy;
	}

	/**
	 * @return Returns the username.
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * @param username
	 *            The username to set.
	 */
	public void setUsername(String username) {
		this.username = username;
	}

	/**
	 * @return Returns the nodeAddress.
	 */
	public String getNodeAddress() {
		return nodeAddress;
	}

	/**
	 * @param nodeAddress
	 *            The nodeAddress to set.
	 */
	public void setNodeAddress(String nodeAddress) {
		this.nodeAddress = nodeAddress;
	}

	/**
	 * @return Returns the nodePort.
	 */
	public int getNodePort() {
		return nodePort;
	}

	/**
	 * @param nodePort
	 *            The nodePort to set.
	 */
	public void setNodePort(int nodePort) {
		this.nodePort = nodePort;
	}

	/**
	 * @return Returns the networkName.
	 */
	public String getNetworkName() {
		return networkName;
	}

	/**
	 * @param networkName
	 *            The networkName to set.
	 */
	public void setNetworkName(String networkName) {
		this.networkName = networkName;
	}

	/**
	 * @return Returns the hopsToLive.
	 */
	public int getHopsToLive() {
		return hopsToLive;
	}

	/**
	 * @param hopsToLive
	 *            The hopsToLive to set.
	 */
	public void setHopsToLive(int hopsToLive) {
		this.hopsToLive = hopsToLive;
	}

	/**
	 * @return Returns the incomingDirectory.
	 */
	public String getIncomingDirectory() {
		return incomingDirectory;
	}

	/**
	 * @param incomingDirectory
	 *            The incomingDirectory to set.
	 */
	public void setIncomingDirectory(String incomingDirectory) {
		this.incomingDirectory = incomingDirectory;
	}

	/**
	 * @return Returns the tempDirectory.
	 */
	public String getTempDirectory() {
		return tempDirectory;
	}

	/**
	 * @param tempDirectory
	 *            The tempDirectory to set.
	 */
	public void setTempDirectory(String tempDirectory) {
		this.tempDirectory = tempDirectory;
	}

	/**
	 * @return Returns the filelistIndex.
	 */
	public int getFilelistIndex() {
		return filelistIndex;
	}

	/**
	 * @param filelistIndex
	 *            The filelistIndex to set.
	 */
	public void setFilelistIndex(int filelistIndex) {
		this.filelistIndex = filelistIndex;
	}

	/**
	 * @return Returns the filelistLastInsertedVersion.
	 */
	public int getFilelistLastInsertedVersion() {
		return filelistLastInsertedVersion;
	}

	/**
	 * @param filelistLastInsertedVersion
	 *            The filelistLastInsertedVersion to set.
	 */
	public void setFilelistLastInsertedVersion(int filelistLastInsertedVersion) {
		this.filelistLastInsertedVersion = filelistLastInsertedVersion;
	}

	/**
	 * @return Returns the sharedDirectories.
	 */
	public SharedDirectory[] getSharedDirectories() {
		return sharedDirectories;
	}

	/**
	 * @param sharedDirectories
	 *            The sharedDirectories to set.
	 */
	public void setSharedDirectories(SharedDirectory[] sharedDirectories) {
		this.sharedDirectories = sharedDirectories;
	}

	/**
	 * @return Returns the announcementService.
	 */
	public AnnouncementService getAnnouncementService() {
		return announcementService;
	}

	/**
	 * @return The client manager service
	 */
	public ClientService getClientService() {
		return clientService;
	}

	/**
	 * @return Returns the clipboardService.
	 */
	public ClipboardService getClipboardService() {
		return clipboardService;
	}

	/**
	 * @return Returns the discoveryService.
	 */
	public DiscoveryService getDiscoveryService() {
		return discoveryService;
	}

	/**
	 * @return Returns the downloadService.
	 */
	public DownloadService getDownloadService() {
		return downloadService;
	}

	/**
	 * @return Returns the engineService.
	 */
	public EngineService getEngineService() {
		return engineService;
	}

	/**
	 * @return Returns the filelistCollectorService.
	 */
	public FilelistCollectorService getFilelistCollectorService() {
		return filelistCollectorService;
	}

	/**
	 * @return Returns the filelistManagerService.
	 */
	public FilelistService getFilelistService() {
		return filelistService;
	}

	/**
	 * @return Returns the filelistService.
	 */
	public FilelistInsertService getFilelistInsertService() {
		return filelistInsertService;
	}

	/**
	 * @return Returns the sharedFileService.
	 */
	public SharedFileService getSharedFileService() {
		return sharedFileService;
	}

	/**
	 * @return Returns the uploadRequestDiscoveryService.
	 */
	public UploadRequestDiscoveryService getUploadRequestDiscoveryService() {
		return uploadRequestDiscoveryService;
	}

	/**
	 * @return Returns the uploadRequestService.
	 */
	public UploadRequestInsertService getUploadRequestInsertService() {
		return uploadRequestInsertService;
	}

	/**
	 * @return Returns the uploadService.
	 */
	public UploadService getUploadService() {
		return uploadService;
	}

	public AbstractService[] getServices() {
		AbstractService[] services = new AbstractService[15];
		services[0] = announcementService;
		services[1] = clientService;
		services[2] = clipboardService;
		services[3] = discoveryService;
		services[4] = downloadService;
		services[5] = engineService;
		services[6] = filelistService;
		services[7] = filelistInsertService;
		services[8] = filelistCollectorService;
		services[11] = sharedFileService;
		services[12] = uploadRequestInsertService;
		services[13] = uploadRequestDiscoveryService;
		services[14] = uploadService;
		return services;
	}

	public void init() {
		super.init();

		configurationListeners = new ArrayList<ConfigurationListener>();
		addServiceListener(this);

		engineService = new EngineService();
		engineService.init();
		engineService.addServiceListener(this);

		announcementService = new AnnouncementService();
		announcementService.init();
		announcementService.addServiceListener(this);
		announcementService.addAnnouncementListener(this);

		clientService = new ClientService();
		clientService.init();
		clientService.addServiceListener(this);
		clientService.addClientListener(this);

		clipboardService = new ClipboardService();
		clipboardService.init();
		clipboardService.addServiceListener(this);
		clipboardService.addClipboardListener(this);

		discoveryService = new DiscoveryService();
		discoveryService.init();
		discoveryService.addServiceListener(this);
		discoveryService.addDiscoveryListener(this);

		downloadService = new DownloadService();
		downloadService.init();
		downloadService.addServiceListener(this);
		downloadService.addDownloadListener(this);

		filelistCollectorService = new FilelistCollectorService();
		filelistCollectorService.init();
		filelistCollectorService.addServiceListener(this);
		filelistCollectorService.addFilelistCollectorListener(this);

		filelistService = new FilelistService();
		filelistService.init();
		filelistService.addServiceListener(this);

		filelistInsertService = new FilelistInsertService();
		filelistInsertService.init();
		filelistInsertService.addServiceListener(this);
		filelistInsertService.addFilelistInsertListener(this);

		sharedFileService = new SharedFileService();
		sharedFileService.init();
		sharedFileService.addServiceListener(this);
		sharedFileService.addSharedFileListener(this);

		uploadRequestDiscoveryService = new UploadRequestDiscoveryService();
		uploadRequestDiscoveryService.init();
		uploadRequestDiscoveryService.addServiceListener(this);
		uploadRequestDiscoveryService.addUploadRequestDiscoveryListener(this);

		uploadRequestInsertService = new UploadRequestInsertService();
		uploadRequestInsertService.init();
		uploadRequestInsertService.addServiceListener(this);
		uploadRequestInsertService.addUploadRequestInsertListener(this);

		uploadService = new UploadService();
		uploadService.init();
		uploadService.addServiceListener(this);
		uploadService.addUploadListener(this);
	}

	public void start() {
		announcementService.setEngine(engineService);
		announcementService.setHopsToLive(hopsToLive);
		discoveryService.setEngine(engineService);
		discoveryService.setHopsToLive(hopsToLive);
		downloadService.setThreadCount(1);
		downloadService.setEngine(engineService);
		downloadService.setHopsToLive(hopsToLive);
		filelistCollectorService.setEngine(engineService);
		filelistCollectorService.setHopsToLive(hopsToLive);
		filelistInsertService.setEngine(engineService);
		filelistInsertService.setHopsToLive(hopsToLive);
		sharedFileService.setEngine(engineService);
		sharedFileService.setHopsToLive(hopsToLive);
		uploadRequestDiscoveryService.setEngine(engineService);
		uploadRequestDiscoveryService.setHopsToLive(hopsToLive);
		uploadRequestInsertService.setEngine(engineService);
		uploadRequestInsertService.setHopsToLive(hopsToLive);
		uploadService.setEngine(engineService);
		uploadService.setHopsToLive(hopsToLive);
		engineService.setFcpNode(new FcpNode(nodeAddress, nodePort));
		engineService.start();
	}

	public void stop() {
		if (announcementService.isRunning()) {
			announcementService.stop();
		}
		if (discoveryService.isRunning()) {
			discoveryService.stop();
		}
		if (clipboardService.isRunning()) {
			clipboardService.stop();
		}
		if (downloadService.isRunning()) {
			downloadService.stop();
		}
		if (uploadService.isRunning()) {
			uploadService.stop();
		}
		if (filelistCollectorService.isRunning()) {
			filelistCollectorService.stop();
		}
		if (filelistInsertService.isRunning()) {
			filelistInsertService.stop();
		}
		if (filelistService.isRunning()) {
			filelistService.stop();
		}
		if (sharedFileService.isRunning()) {
			sharedFileService.stop();
		}
		if (uploadRequestDiscoveryService.isRunning()) {
			uploadRequestDiscoveryService.stop();
		}
		if (uploadRequestInsertService.isRunning()) {
			uploadRequestInsertService.stop();
		}
		if (engineService.isRunning()) {
			engineService.stop();
		}

		super.stop();
	}

	public void destroy() {
		announcementService.destroy();
		discoveryService.destroy();
		clipboardService.destroy();
		downloadService.destroy();
		uploadService.destroy();
		engineService.destroy();
		filelistCollectorService.destroy();
		filelistService.destroy();
		filelistInsertService.destroy();
		sharedFileService.destroy();
		uploadRequestDiscoveryService.destroy();
		uploadRequestInsertService.destroy();
		super.destroy();
	}

	private void startSharedFileService() {
		if (!sharedFileService.isRunning()) {
			sharedFileService.setSharedDirectories(sharedDirectories);
			sharedFileService.start();
		}
	}

	private void startFilelistInsertService() {
		if (!filelistInsertService.isRunning()) {
			filelistInsertService.setClientPrivateAddress(thisClient.getPrivateAddress());
			filelistInsertService.setNetworkName(networkName);
			filelistInsertService.start();
		}
	}

	private void startFilelistService() {
		if (!filelistService.isRunning()) {
			filelistService.start();
		}
	}

	private void startAnnouncementService() {
		if (!announcementService.isRunning()) {
			announcementService.setClient(thisClient);
			announcementService.setNetworkName(networkName);
			announcementService.start();
			announcementService.insertNow(filelistIndex);
		}
	}

	private void startClipboardService() {
		if (!clipboardService.isRunning()) {
			clipboardService.start();
		}
	}

	private void startDiscoveryService() {
		if (!discoveryService.isRunning()) {
			discoveryService.setNetworkName(networkName);
			discoveryService.start();
		}
	}

	private void startDownloadService() {
		if (!downloadService.isRunning()) {
			downloadService.setTempDirectory(tempDirectory);
			downloadService.setIncomingDirectory(incomingDirectory);
			downloadService.start();
		}
	}

	private void startFilelistCollectorService() {
		if (!filelistCollectorService.isRunning()) {
			filelistCollectorService.start();
		}
	}

	private void startUploadRequestInsertService() {
		if (!uploadRequestInsertService.isRunning()) {
			uploadRequestInsertService.setClient(thisClient);
			uploadRequestInsertService.setNetworkName(networkName);
			uploadRequestInsertService.start();
		}
	}

	private void startUploadRequestDiscoveryService() {
		if (!uploadRequestDiscoveryService.isRunning()) {
			uploadRequestDiscoveryService.setNetworkName(networkName);
			uploadRequestDiscoveryService.start();
		}
	}

	private void startUploadService() {
		if (!uploadService.isRunning()) {
			uploadService.setSharedFileService(sharedFileService);
			uploadService.start();
		}
	}

	private void checkForFatalException(Throwable throwable) {
		if (throwable == null)
			return;
		if (throwable instanceof FcpException) {
			if ("No space left on device".equals(throwable.getMessage())) {
				stop();
			}
		}
		if (throwable instanceof IOException) {
			if ("Connection refused".equals(throwable.getMessage())) {
				stop();
			}
		}
	}

	//
	// INTERFACE ServiceListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void serviceStarted(Service service) {
		if (service.equals(engineService)) {
			// now check config
			if (publicKey == null) {
				try {
					String[] keys = engineService.generateSecretKey();
					privateKey = keys[0];
					publicKey = keys[1];
					entropy = keys[2];
					fireConfigurationChanged();
				} catch (FcpException fe1) {
					fireServiceAborted(fe1);
					stop();
					return;
				} catch (IOException ioe1) {
					fireServiceAborted(ioe1);
					stop();
					return;
				} catch (EngineStoppedException ese1) {
					fireServiceAborted(ese1);
					stop();
					return;
				}
			}
			thisClient = new Client(publicKey, username, "freeMule " + Version.getVersion(), "SSK@" + publicKey + "PAgM," + entropy, "SSK@" + privateKey + "," + entropy, filelistIndex, System.currentTimeMillis());
			super.start();
		} else if (service.equals(this)) {
			startSharedFileService();
		} else if (service.equals(sharedFileService)) {
			startFilelistInsertService();
		} else if (service.equals(filelistInsertService)) {
			startAnnouncementService();
		} else if (service.equals(announcementService)) {
			startDiscoveryService();
		} else if (service.equals(discoveryService)) {
			startClipboardService();
		} else if (service.equals(clipboardService)) {
			startFilelistService();
		} else if (service.equals(filelistService)) {
			startFilelistCollectorService();
		} else if (service.equals(filelistCollectorService)) {
			startUploadService();
		} else if (service.equals(uploadService)) {
			startUploadRequestInsertService();
		} else if (service.equals(uploadRequestInsertService)) {
			startDownloadService();
		} else if (service.equals(downloadService)) {
			FreenetFile[] downloadFiles = downloadService.getDownloads();
			for (FreenetFile downloadFile: downloadFiles) {
				uploadRequestInsertService.addUploadRequest(downloadFile.getKey(), downloadFile.getBlockList(BlockStatus.notAvailable));
			}
			startUploadRequestDiscoveryService();
		} else if (service.equals(uploadRequestDiscoveryService)) {
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceAborted(Service service, Throwable throwable) {
		checkForFatalException(throwable);
		if (service.equals(engineService)) {
			fireServiceAborted(throwable);
			stop();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceStopped(Service service) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceThreadStarted(Service service, Thread thread) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceThreadUpdated(Service service, Thread thread) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceThreadAborted(Service service, Thread thread, Throwable throwable) {
		checkForFatalException(throwable);
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceThreadStopped(Service service, Thread thread) {
	}

	//
	// INTERFACE AnnouncementListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void announcementInsertStarted(String key) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void announcementInsertFailed(String key, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void announcementInsertComplete(String key) {
	}

	//
	// INTERFACE DiscoveryListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void discoveryStarted(String key) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void discoveryFailed(String key, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void discoveryComplete(String key, Client remoteClient) {
		clientService.addClient(remoteClient);
	}

	//
	// INTERFACE DownloadListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void downloadAdded(FreenetFile freenetFile) {
		uploadRequestInsertService.addUploadRequest(freenetFile.getKey(), freenetFile.getBlockCount());
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadRemoved(FreenetFile freenetFile) {
		uploadRequestInsertService.removeUploadRequest(freenetFile.getKey());
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadStarted(FreenetFile freenetFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadStopped(FreenetFile freenetFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadCompleting(FreenetFile freenetFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadCompletionFailed(FreenetFile freenetFile, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadComplete(FreenetFile freenetFile) {
		uploadRequestInsertService.removeUploadRequest(freenetFile.getKey());
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestDownloadStarted(FreenetFile freenetFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestDownloadFailed(FreenetFile freenetFile, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestDownloadComplete(FreenetFile freenetFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockDownloadStarted(FreenetFile freenetFile, int blockNumber) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockDownloadFailed(FreenetFile freenetFile, int blockNumber, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockDownloadComplete(FreenetFile freenetFile, int blockNumber) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadsStarted(String tempDir) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadsFailed(String tempDir, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadsComplete(String tempDir) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadStarted(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadComplete(String filename, FreenetFile freenetFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadFailed(String filename, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadsStarted(String tempDir) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadsFailed(String tempDir, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadsComplete(String tempDir) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadStarted(FreenetFile freenetFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadComplete(FreenetFile freenetFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadFailed(FreenetFile freenetFile, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveBlockStarted(FreenetFile freenetFile, int blockNumber) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveBlockFailed(FreenetFile freenetFile, int blockNumber, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveBlockComplete(FreenetFile freenetFile, int blockNumber) {
	}

	//
	// INTERFACE FilelistCollectorListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void filelistCollectingStarted(Client remoteClient, String key) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void filelistCollectingFailed(Client remoteClient, String key, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void filelistCollectingComplete(Client remoteClient, String key, SharedFile[] sharedFiles) {
		filelistService.addClientFilelist(remoteClient, sharedFiles);
	}

	//
	// INTERFACE FilelistInsertListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void filelistInsertStarted(String key, int index) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void filelistInsertFailed(String key, int index, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void filelistInsertComplete(String key, int newIndex) {
		filelistLastInsertedVersion = newIndex;
		thisClient.setFilelistIndex(newIndex);
		thisClient.setTimestamp(System.currentTimeMillis());
		fireConfigurationChanged();
		announcementService.insertNow(newIndex);
	}

	//
	// INTERFACE SharedFileListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileLoadingStarted(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileLoadingFailed(String filename, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileLoadingComplete(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileUnsharedFile(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileUnsharedDirectory(String directory) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileSavingStarted(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileSavingFailed(String filename, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileSavingComplete(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileScanStarted() {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileScanFailed(Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileScanComplete() {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileDirectoryScanStarted(String directory) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileDirectoryScanFailed(String directory, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileDirectoryScanComplete(String directory) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileHashingStarted(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileHashingFailed(String filename, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileHashingComplete(String filename, String key, long filelength, long duration) {
	}

	public void sharedFileHashingProgress(String filename, int blockIndex, int blockCount) {

	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileListChanged() {
		filelistIndex++;
		filelistInsertService.addSharedFiles(filelistIndex, sharedFileService.getSharedFiles());
		fireConfigurationChanged();
	}

	//
	// INTERFACE UploadRequestDiscoveryListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void uploadRequestDiscoveryStarted(String key) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void uploadRequestDiscoveryFailed(String key, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * If an {@link UploadRequest upload request} is found the keys of it are searched for a match in the
	 * {@link SharedFileService shared file service}. If we have one of the keys that was requested and we already know the client that made this
	 * request we add this request to the {@link UploadService upload service}.
	 * 
	 * @param key
	 *            The key of the {@link UploadRequest upload request}
	 * @param uploadRequest
	 *            The {@link UploadRequest upload request} that was found
	 */
	public void uploadRequestDiscoveryComplete(String key, UploadRequest uploadRequest) {
		Client remoteClient = clientService.getClient(uploadRequest.getClientId());
		if (remoteClient != null) {
			uploadService.addUploadRequest(remoteClient, uploadRequest);
		}
	}

	//
	// INTERFACE UploadRequestInsertListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void uploadRequestInsertStarted(String key) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void uploadRequestInsertFailed(String key, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void uploadRequestInsertComplete(String key) {
	}

	//
	// INTERFACE ClientListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void clientAdded(Client remoteClient) {
		filelistCollectorService.addClient(remoteClient);
	}

	/**
	 * {@inheritDoc}
	 */
	public void clientUpdated(Client remoteClient) {
		filelistCollectorService.addClient(remoteClient);
	}

	/**
	 * {@inheritDoc}
	 */
	public void clientRemoved(Client remoteClient) {
		filelistService.removeClientFilelist(remoteClient);
	}

	//
	// INTERFACE ClipboardListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void urlFound(String url) {
		String key = url.trim();
		String name = String.valueOf(key.hashCode()) + ".bin";
		int slashPosition = key.indexOf('/');
		if (slashPosition > 0) {
			int lastSlashPosition = key.lastIndexOf('/');
			name = key.substring(lastSlashPosition + 1);
			key = key.substring(0, slashPosition);
		}
		downloadService.addDownload(key, name);
	}

	//
	// INTERFACE UploadListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void uploadAdded(UploadFile uploadFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void uploadUpdated(UploadFile uploadFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void uploadRemoved(UploadFile uploadFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void uploadComplete(UploadFile uploadFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockUploadStarted(UploadFile uploadFile, int blockIndex) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockUploadFailed(UploadFile uploadFile, int blockIndex, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockUploadComplete(UploadFile uploadFile, int blockIndex) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestUploadStarted(UploadFile uploadFile) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestUploadFailed(UploadFile uploadFile, Exception exception) {
		checkForFatalException(exception);
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestUploadComplete(UploadFile uploadFile) {
	}

}
