/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.application;

import java.util.EventListener;

/**
 * Interface for listeners that want to be notified on service events.
 * 
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id:ServiceListener.java 220 2006-03-06 14:46:23Z bombe $
 */
public interface ServiceListener extends EventListener {

	/**
	 * Notifies listeners that a service has been started. If a service has more
	 * than one thread this notification will be sent as soon as the last
	 * required thread has been started.
	 * 
	 * @param service
	 *            The service that has been started
	 */
	public void serviceStarted(Service service);

	/**
	 * Notifies listeners that a service has been aborted.
	 * 
	 * @param service
	 *            The service that has been aborted
	 * @param throwable
	 *            The {@link Throwable throwable} that caused the abortion
	 */
	public void serviceAborted(Service service, Throwable throwable);

	/**
	 * Notifies listeners that a service has stopped. If a service has more than
	 * one thread this notification will be sent after the last thread has
	 * stopped.
	 * 
	 * @param service
	 *            The service that has been stopped
	 */
	public void serviceStopped(Service service);

	/**
	 * Notifies listeners that a service thread has been started. This
	 * notification will be sent for every thread that is started.
	 * 
	 * @param service
	 *            The service of the thread
	 * @param thread
	 *            The thread that has been started
	 */
	public void serviceThreadStarted(Service service, Thread thread);

	public void serviceThreadUpdated(Service service, Thread thread);
	
	/**
	 * Notifies listeners that a service thread has been aborted
	 * 
	 * @param service
	 *            The service of the thread
	 * @param thread
	 *            The thread that has been aborted
	 * @param throwable
	 *            The {@link Throwable throwable} that caused the abortion
	 */
	public void serviceThreadAborted(Service service, Thread thread, Throwable throwable);

	/**
	 * Notifies listeners that a service thread has been stopped. This
	 * notification will be sent for every thread that has been stopped.
	 * 
	 * @param service
	 *            The service of the thread
	 * @param thread
	 *            The thread that has been stopped
	 */
	public void serviceThreadStopped(Service service, Thread thread);

}
