/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.gui;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import de.todesbaum.freemule.application.SharedDirectory;



/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id:DirectoryTableModel.java 221 2006-03-06 14:46:49Z bombe $
 */
public class DirectoryTableModel extends AbstractTableModel {

	private List<SharedDirectory> shownDirectories = new ArrayList<SharedDirectory>();
	
	public void clear() {
		shownDirectories.clear();
		fireTableDataChanged();
	}
	
	public void addSharedDirectory(SharedDirectory sharedDirectory) {
		shownDirectories.add(sharedDirectory.clone());
		fireTableRowsInserted(size(), size());
	}
	
	public void removeSharedDirectory(int index) {
		shownDirectories.remove(index);
		fireTableRowsDeleted(index, index);
	}
	
	public int size() {
		return shownDirectories.size();
	}
	
	public SharedDirectory getSharedDirectory(int index) {
		return shownDirectories.get(index);
	}
	
	public SharedDirectory[] getSharedDirectories() {
		SharedDirectory[] result = new SharedDirectory[shownDirectories.size()];
		for (int index = 0, size = shownDirectories.size(); index < size; index++) {
			result[index] = shownDirectories.get(index);
		}
		return result;
	}
	
	/**
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return shownDirectories.size();
	}

	/**
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return 2;
	}

	/**
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	public String getColumnName(int columnIndex) {
		return new String[] { "Recursive", "Directory"  }[columnIndex];
	}

	/**
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 */
	public Class< ? > getColumnClass(int columnIndex) {
		return new Class<?>[] { Boolean.class, String.class }[columnIndex];
	}

	/**
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return (columnIndex == 0);
	}

	/**
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		SharedDirectory shownDirectory = shownDirectories.get(rowIndex);
		return (columnIndex == 1) ? shownDirectory.getDirectory() : Boolean.valueOf(shownDirectory.isRecursive());
	}

	/**
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		shownDirectories.get(rowIndex).setRecursive(((Boolean) aValue).booleanValue());
		fireTableCellUpdated(rowIndex, columnIndex);
	}

}
