/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.gui;

import javax.swing.table.AbstractTableModel;

import de.todesbaum.freemule.application.Client;
import de.todesbaum.freemule.application.ClientListener;
import de.todesbaum.freemule.application.ClientService;
import de.todesbaum.freemule.application.FilelistListener;
import de.todesbaum.freemule.application.FilelistService;
import de.todesbaum.freemule.application.SharedFile;


/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id:KnownClientsTableModel.java 221 2006-03-06 14:46:49Z bombe $
 */
public class KnownClientsTableModel extends AbstractTableModel implements ClientListener, FilelistListener {

	private static final String[] columnNames = new String[] { "Username", "Client ID", "Version", "Files" };
	private static final Class[] columnClasses = new Class[] { String.class, String.class, String.class, Integer.class };

	private Client[] clients;
	private ClientService clientService;
	private FilelistService filelistService;

	public void setClientService(ClientService clientService) {
		if (this.clientService != null) {
			this.clientService.removeClientListener(this);
		}
		this.clientService = clientService;
		if (this.clientService != null) {
			this.clientService.addClientListener(this);
		}
		updateClients();
	}

	public void setFilelistService(FilelistService filelistService) {
		if (this.filelistService != null) {
			this.filelistService.removeFilelistListener(this);
		}
		this.filelistService = filelistService;
		if (this.filelistService != null) {
			this.filelistService.addFilelistListener(this);
		}
		updateClients();
	}

	private void updateClients() {
		clients = clientService.getClients();
		fireTableDataChanged();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class< ? > getColumnClass(int columnIndex) {
		return columnClasses[columnIndex];
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getColumnName(int column) {
		return columnNames[column];
	}

	/**
	 * {@inheritDoc}
	 */
	public int getRowCount() {
		if (clients == null) {
			return 0;
		}
		return clients.length;
	}

	/**
	 * {@inheritDoc}
	 */
	public int getColumnCount() {
		return columnNames.length;
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getValueAt(int row, int column) {
		if (clients == null) {
			return null;
		}
		Client client = clients[row];
		switch (column) {
		case -1:
			return client;
		case 0:
			return client.getName();
		case 1:
			return client.getClientId();
		case 2:
			return client.getVersion();
		case 3:
			if (filelistService == null) {
				return 0;
			}
			SharedFile[] clientFilelist = filelistService.getClientFilelist(client);
			if (clientFilelist == null) {
				return 0;
			}
			return clientFilelist.length;
		}
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public void clientAdded(Client remoteClient) {
		updateClients();
	}

	/**
	 * {@inheritDoc}
	 */
	public void clientUpdated(Client remoteClient) {
		updateClients();
	}

	/**
	 * {@inheritDoc}
	 */
	public void clientRemoved(Client remoteClient) {
		updateClients();
	}

	/**
	 * {@inheritDoc}
	 */
	public void filelistAdded(Client remoteClient, SharedFile[] sharedFiles) {
		updateClients();
	}

	/**
	 * {@inheritDoc}
	 */
	public void filelistUpdated(Client remoteClient, SharedFile[] sharedFiles) {
		updateClients();
	}

	/**
	 * {@inheritDoc}
	 */
	public void filelistRemoved(Client remoteClient) {
		updateClients();
	}

}
