/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

public abstract class OkayCancelDialog extends JDialog {

	public static final int OKAY_BUTTON = 1;
	public static final int CANCEL_BUTTON = 2;

	private JComponent contentPane;

	private Action actionOkay;
	private Action actionCancel;

	public OkayCancelDialog(JFrame frame, String title) {
		this(frame, title, OKAY_BUTTON | CANCEL_BUTTON);
	}

	public OkayCancelDialog(JFrame frame, String title, int buttons) {
		super(frame, title, true);

		actionOkay = new AbstractAction("Okay") {

			public void actionPerformed(ActionEvent actionEvent) {
				okayAction();
			}
		};

		actionCancel = new AbstractAction("Cancel") {

			public void actionPerformed(ActionEvent actionEvent) {
				cancelAction();
			}
		};

		contentPane = new JPanel(new BorderLayout(12, 12));
		contentPane.setBorder(new EmptyBorder(12, 12, 12, 12));
		JPanel innerPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING, 12, 12));
		innerPanel.setBorder(new EmptyBorder(-12, -12, -12, -12));
		if ((buttons & CANCEL_BUTTON) > 0) {
			JButton cancelButton = new JButton(actionCancel);
			innerPanel.add(cancelButton);
			getRootPane().setDefaultButton(cancelButton);
		}
		if ((buttons & OKAY_BUTTON) > 0) {
			JButton okayButton = new JButton(actionOkay);
			innerPanel.add(okayButton);
			getRootPane().setDefaultButton(okayButton);
		}
		contentPane.add(innerPanel, BorderLayout.PAGE_END);
		super.setContentPane(contentPane);

		addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent windowEvent) {
				cancelAction();
			}
		});
	}

	@Override
	public void pack() {
		super.pack();
		Dimension size = getSize();
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation((screen.width - size.width) / 2, (screen.height - size.height) / 2);
	}

	@Override
	public void setContentPane(Container container) {
		contentPane.add(container, BorderLayout.CENTER);
	}

	protected void okayAction() {
		setVisible(false);
	}

	protected void cancelAction() {
		setVisible(false);
	}

}
