/*
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.gui;

import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.text.ParseException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.JSpinner.DefaultEditor;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeListener;
import javax.swing.text.JTextComponent;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden </a>
 * @version $Id:PreferencesDialog.java 221 2006-03-06 14:46:49Z bombe $
 */
public class PreferencesDialog extends OkayCancelDialog {

	private Action tempDirectoryBrowseAction;
	private Action incomingDirectoryBrowseAction;
	private Action restoreDefaultNetworkNameAction;

	private JTextField clientNameTextField;
	private JTextField clientIdTextField;
	protected JTextField tempDirectoryTextField;
	protected JTextField incomingDirectoryTextField;
	protected JTextField networkNameTextField;
	protected JSpinner hopsToLiveSpinner;

	private String clientName;
	private String tempDirectory;
	private String incomingDirectory;
	private String networkName;
	private int hopsToLive;

	public PreferencesDialog(JFrame owner) {
		super(owner, "Preferences");
		createActions();
		initialize();
		pack();
		// setSize(getWidth() * 2, getHeight());
	}

	public String getUsername() {
		return clientName;
	}

	public void setUsername(String clientName) {
		this.clientName = clientName;
		clientNameTextField.setText(clientName);
	}

	public void setClientID(String clientID) {
		clientIdTextField.setText(clientID);
	}

	public String getIncomingDirectory() {
		return incomingDirectory;
	}

	public void setIncomingDirectory(String incomingDirectory) {
		this.incomingDirectory = incomingDirectory;
		incomingDirectoryTextField.setText(incomingDirectory);
	}

	public String getTempDirectory() {
		return tempDirectory;
	}

	public void setTempDirectory(String tempDirectory) {
		this.tempDirectory = tempDirectory;
		tempDirectoryTextField.setText(tempDirectory);
	}

	/**
	 * @return Returns the networkName.
	 */
	public String getNetworkName() {
		return networkName;
	}

	/**
	 * @param networkName
	 *            The networkName to set.
	 */
	public void setNetworkName(String networkName) {
		this.networkName = networkName;
		networkNameTextField.setText(networkName);
	}

	/**
	 * @return Returns the hopsToLive.
	 */
	public int getHopsToLive() {
		return hopsToLive;
	}

	/**
	 * @param hopsToLive
	 *            The hopsToLive to set.
	 */
	public void setHopsToLive(int hopsToLive) {
		this.hopsToLive = hopsToLive;
		hopsToLiveSpinner.getModel().setValue(hopsToLive);
	}

	private void initialize() {
		setContentPane(createContentPane());
	}

	private Container createContentPane() {
		JPanel contentPane = new JPanel(new LineLayout(6));

		/* identity settings */
		JLabel identityLabel = new JLabel("<html><b>Identity</b></html>");
		contentPane.add(identityLabel);

		GridBagLayout identityTableLayout = new GridBagLayout();
		JPanel identityTable = new JPanel(identityTableLayout);
		identityTable.setBorder(new EmptyBorder(0, 24, 0, 0));
		contentPane.add(identityTable);

		JLabel clientNameLabel = new JLabel("Username:");
		clientNameLabel.setFont(clientNameLabel.getFont().deriveFont(Font.PLAIN));
		clientNameLabel.setDisplayedMnemonic('u');
		clientNameTextField = new JTextField();
		clientNameLabel.setLabelFor(clientNameTextField);
		identityTable.add(clientNameLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		identityTable.add(clientNameTextField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 6, 0, 0), 0, 0));

		JLabel clientIdLabel = new JLabel("ID:");
		clientIdLabel.setFont(clientIdLabel.getFont().deriveFont(Font.PLAIN));
		clientIdTextField = new JTextField();
		clientIdTextField.setEditable(false);
		identityTable.add(clientIdLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(6, 0, 0, 0), 0, 0));
		identityTable.add(clientIdTextField, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 0), 0, 0));

		/* directory settings */
		JLabel directoryLabel = new JLabel("<html><b>Directory</b></html>");
		directoryLabel.setBorder(new EmptyBorder(12, 0, 0, 0));
		contentPane.add(directoryLabel);

		GridBagLayout directoryTableLayout = new GridBagLayout();
		JPanel directoryPanel = new JPanel(directoryTableLayout);
		directoryPanel.setBorder(new EmptyBorder(0, 24, 0, 0));
		contentPane.add(directoryPanel);

		JLabel tempDirectoryLabel = new JLabel("Temporary directory:");
		tempDirectoryLabel.setFont(tempDirectoryLabel.getFont().deriveFont(Font.PLAIN));
		tempDirectoryLabel.setDisplayedMnemonic('t');
		tempDirectoryTextField = new JTextField(20);
		tempDirectoryLabel.setLabelFor(tempDirectoryTextField);
		JButton tempBrowseButton = new JButton(tempDirectoryBrowseAction);
		directoryPanel.add(tempDirectoryLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		directoryPanel.add(tempDirectoryTextField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.HORIZONTAL, new Insets(0, 6, 0, 6), 0, 0));
		directoryPanel.add(tempBrowseButton, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

		JLabel incomingDirectoryLabel = new JLabel("Incoming directory:");
		incomingDirectoryLabel.setFont(incomingDirectoryLabel.getFont().deriveFont(Font.PLAIN));
		incomingDirectoryLabel.setDisplayedMnemonic('i');
		incomingDirectoryTextField = new JTextField();
		incomingDirectoryLabel.setLabelFor(incomingDirectoryTextField);
		JButton incomingBrowseButton = new JButton(incomingDirectoryBrowseAction);
		directoryPanel.add(incomingDirectoryLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(6, 0, 0, 0), 0, 0));
		directoryPanel.add(incomingDirectoryTextField, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 6), 0, 0));
		directoryPanel.add(incomingBrowseButton, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(6, 0, 0, 0), 0, 0));

		/* network settings */
		JLabel networkLabel = new JLabel("<html><b>Network Settings</b></html>");
		networkLabel.setBorder(new EmptyBorder(12, 0, 0, 0));
		contentPane.add(networkLabel);

		GridBagLayout networkTableLayout = new GridBagLayout();
		JPanel networkTable = new JPanel(networkTableLayout);
		networkTable.setBorder(new EmptyBorder(0, 24, 0, 0));
		contentPane.add(networkTable);

		JLabel warningLabel = new JLabel("These values should only be changed if you really know what you are doing!", IconLoader.loadIcon("/img/warning.png"), SwingConstants.LEADING);
		warningLabel.setIconTextGap(12);
		warningLabel.setAlignmentY(0.0f);
		warningLabel.setAlignmentX(0.0f);
		warningLabel.setFont(warningLabel.getFont().deriveFont(Font.PLAIN));
		warningLabel.setForeground(Color.RED);
		networkTable.add(warningLabel, new GridBagConstraints(0, 0, 3, 1, 1.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

		JLabel nameLabel = new JLabel("Network name:");
		nameLabel.setFont(nameLabel.getFont().deriveFont(Font.PLAIN));
		nameLabel.setDisplayedMnemonic('n');
		networkNameTextField = new JTextField();
		nameLabel.setLabelFor(networkNameTextField);
		networkTable.add(nameLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 1.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(6, 0, 0, 0), 0, 0));
		networkTable.add(networkNameTextField, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0, GridBagConstraints.LINE_START, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 0), 0, 0));
		networkTable.add(new JButton(restoreDefaultNetworkNameAction), new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 0));

		JLabel hopsToLiveLabel = new JLabel("Hops to live:");
		hopsToLiveLabel.setFont(hopsToLiveLabel.getFont().deriveFont(Font.PLAIN));
		hopsToLiveLabel.setDisplayedMnemonic('h');
		hopsToLiveSpinner = new JSpinner(new SpinnerNumberModel(hopsToLive, 0, 25, 1));
		((DefaultEditor) hopsToLiveSpinner.getEditor()).getTextField().setColumns(6);
		hopsToLiveLabel.setLabelFor(hopsToLiveSpinner);
		networkTable.add(hopsToLiveLabel, new GridBagConstraints(0, 2, 1, 1, 0.0, 1.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(6, 0, 0, 0), 0, 0));
		networkTable.add(hopsToLiveSpinner, new GridBagConstraints(1, 2, 2, 1, 1.0, 1.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 0));
		return contentPane;
	}

	private void createActions() {
		tempDirectoryBrowseAction = new AbstractAction("Browse") {

			public void actionPerformed(ActionEvent actionEvent) {
				String directory = browseForDirectory(tempDirectoryTextField.getText());
				if (directory != null) {
					tempDirectoryTextField.setText(directory);
				}
			}
		};
		tempDirectoryBrowseAction.putValue(Action.SHORT_DESCRIPTION, "Change temporary directory");
		tempDirectoryBrowseAction.putValue(Action.MNEMONIC_KEY, KeyEvent.VK_B);

		incomingDirectoryBrowseAction = new AbstractAction("Browse") {

			public void actionPerformed(ActionEvent actionEvent) {
				String directory = browseForDirectory(incomingDirectoryTextField.getText());
				if (directory != null) {
					incomingDirectoryTextField.setText(directory);
				}
			}
		};
		incomingDirectoryBrowseAction.putValue(Action.SHORT_DESCRIPTION, "Change incoming directory");
		incomingDirectoryBrowseAction.putValue(Action.MNEMONIC_KEY, KeyEvent.VK_R);

		restoreDefaultNetworkNameAction = new AbstractAction("Default") {

			public void actionPerformed(ActionEvent actionEvent) {
				networkNameTextField.setText("freeMule");
			}
		};
		restoreDefaultNetworkNameAction.putValue(Action.SHORT_DESCRIPTION, "Restores the default network name");
		restoreDefaultNetworkNameAction.putValue(Action.MNEMONIC_KEY, KeyEvent.VK_D);
	}

	@Override
	protected void cancelAction() {
		setVisible(false);
	}

	@Override
	protected void okayAction() {
		clientName = clientNameTextField.getText().trim();
		incomingDirectory = incomingDirectoryTextField.getText();
		tempDirectory = tempDirectoryTextField.getText();
		networkName = networkNameTextField.getText().trim();
		hopsToLive = (Integer) hopsToLiveSpinner.getValue();
		setVisible(false);
	}

	protected String browseForDirectory(String directory) {
		JFileChooser fileChooser = new JFileChooser(directory);
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int answer = fileChooser.showDialog(this, "Choose");
		if (answer == JFileChooser.APPROVE_OPTION) {
			return fileChooser.getSelectedFile().getPath();
		}
		return null;
	}

}
