/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.freemule.util;

import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.TestCase;

import org.w3c.dom.Document;


/**
 * @author David Roden &lt;dr@todesbaum.dyndns.org&gt;
 * @version $Id$
 */
public class XMLTest extends TestCase {

	private String testXmlString;
	private byte[] testByteArray;
	private Document testDocument;
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		testXmlString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
				"<xml-test>" +
				"<element>" +
				"<other-element>value" +
				"</other-element>" +
				"<more>" +
				"<even-more>4711</even-more>" +
				"</more>" +
				"</element>" +
				"</xml-test>";
		testByteArray = testXmlString.getBytes();
		DocumentBuilder documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		testDocument = documentBuilder.parse(new ByteArrayInputStream(testByteArray));
	}
	
	/*
	 * Test method for 'freemule.util.XML.createDocument()'
	 */
	public void testCreateDocument() {
		Document document = XML.createDocument();
		assertNotNull("createDocument result", document);
	}

	/*
	 * Test method for 'freemule.util.XML.transformToByteArray(Document)'
	 */
	public void testTransformToByteArray() throws Exception {
		byte[] byteArray = XML.transformToByteArray(testDocument);
		assertEquals("byte array length", 154, byteArray.length);
		assertEquals("byte array content", testXmlString, new String(byteArray));
	}

	/*
	 * Test method for 'freemule.util.XML.transformToDocument(byte[])'
	 */
	public void testTransformToDocument() {
		Document document = XML.transformToDocument(testByteArray);
		assertEquals("document node name", "#document", document.getNodeName());
		assertEquals("document child count", 1, document.getChildNodes().getLength());
		assertEquals("document element", "xml-test", document.getFirstChild().getNodeName());
	}

}
