/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.util.freenet.fcp;

import java.util.EventListener;

/**
 * Interface for all clients who want to be informed about FCP Restarting and
 * Pending events. This is a purely optional operation; all FCP transactions
 * will complete normally even if this interface is not used.
 * 
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpEventListener.java 218 2006-03-06 14:43:35Z  $
 */
public interface FcpEventListener extends EventListener {

	/**
	 * Notifies the listener that an FCP event was received from the Freenet
	 * {@link FcpNode node}.
	 * 
	 * @param source
	 *            The {@link FcpCommand} that is currently executed
	 * @param message
	 *            The event that was received from the {@link FcpNode node}
	 */
	public void fcpGotMessage(FcpCommand source, String message);

}
