/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.util.freenet.fcp;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpFecSegmentFile.java 218 2006-03-06 14:43:35Z  $
 */
public class FcpFecSegmentFile extends FcpCommand {

	private long fileLength;
	private String algoName;
	private List segmentHeaders = new ArrayList();

	public FcpFecSegmentFile(long fileLength) {
		this(fileLength, "OnionFEC_a_1_2");
	}

	public FcpFecSegmentFile(long fileLength, String algoName) {
		this.fileLength = fileLength;
		this.algoName = algoName;
	}

	/**
	 * @see de.todesbaum.util.freenet.fcp.FcpCommand#getName()
	 */
	public String getName() {
		return "FECSegmentFile";
	}
	
	public int getSegmentHeaderCount() {
		return segmentHeaders.size();
	}
	
	public SegmentHeader getSegmentHeader(int index) {
		return (SegmentHeader) segmentHeaders.get(index);
	}

	/**
	 * @see de.todesbaum.util.freenet.fcp.FcpCommand#writeCommand(java.io.OutputStream)
	 */
	public boolean writeCommand(OutputStream outputStream) throws IOException {
		outputStream.write(("AlgoName=" + algoName + "\r\n").getBytes());
		outputStream.write(("FileLength=" + encodeHex(fileLength) + "\r\n").getBytes());
		return true;
	}

	/**
	 * @see de.todesbaum.util.freenet.fcp.FcpCommand#handleFcpMessage(de.todesbaum.util.freenet.fcp.FcpMessage)
	 */
	public void handleFcpMessage(FcpMessage fcpMessage) throws FcpException {
		if (fcpMessage.getName().equals("SegmentHeader")) {
			SegmentHeader segmentHeader = new SegmentHeader();
			segmentHeader.setFecAlgorithm(fcpMessage.getParameter("FECAlgorithm"));
			segmentHeader.setFileLength(parseHex(fcpMessage.getParameter("FileLength")));
			segmentHeader.setOffset(parseHex(fcpMessage.getParameter("Offset")));
			segmentHeader.setBlockCount((int) parseHex(fcpMessage.getParameter("BlockCount")));
			segmentHeader.setBlockSize((int) parseHex(fcpMessage.getParameter("BlockSize")));
			segmentHeader.setCheckBlockCount((int) parseHex(fcpMessage.getParameter("CheckBlockCount")));
			segmentHeader.setCheckBlockSize((int) parseHex(fcpMessage.getParameter("CheckBlockSize")));
			segmentHeader.setSegmentCount((int) parseHex(fcpMessage.getParameter("Segments")));
			segmentHeader.setSegment((int) parseHex(fcpMessage.getParameter("SegmentNum")));
			segmentHeader.setBlocksRequired((int) parseHex(fcpMessage.getParameter("BlocksRequired")));
			segmentHeaders.add(segmentHeader);
			complete = segmentHeader.getSegment() == (segmentHeader.getSegmentCount() - 1);
		}
	}

}
