/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package de.todesbaum.util.freenet.fcp;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;


/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: FcpFecSegmentSplitFile.java 218 2006-03-06 14:43:35Z  $
 */
public class FcpFecSegmentSplitFile extends FcpCommand {

	private byte[] metadata;
	private List segmentHeaders = new ArrayList();
	private List blockMaps = new ArrayList();
	private SegmentHeader lastSegmentHeader;
	
	/**
	 * 
	 */
	public FcpFecSegmentSplitFile(byte[] metadata) {
		this.metadata = metadata;
	}
	
	public int getSegmentHeaderCount() {
		return segmentHeaders.size();
	}
	
	public SegmentHeader getSegmentHeader(int index) {
		return (SegmentHeader) segmentHeaders.get(index);
	}
	
	public int getBlockMapCount() {
		return blockMaps.size();
	}
	
	public BlockMap getBlockMap(int index) {
		return (BlockMap) blockMaps.get(index);
	}

	/**
	 * {@inheritDoc}
	 */
	public String getName() {
		return "FECSegmentSplitFile";
	}

	/**
	 * {@inheritDoc}
	 */
	protected boolean writeCommand(OutputStream outputStream) throws IOException {
		outputStream.write(("DataLength=" + encodeHex(metadata.length) + "\r\n").getBytes());
		outputStream.write(("Data\r\n").getBytes());
		outputStream.write(metadata);
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	protected void handleFcpMessage(FcpMessage fcpMessage) throws FcpException {
		if ("SegmentHeader".equals(fcpMessage.getName())) {
			lastSegmentHeader = new SegmentHeader();
			segmentHeaders.add(lastSegmentHeader);
			lastSegmentHeader.setFecAlgorithm(fcpMessage.getParameter("FECAlgoName"));
			lastSegmentHeader.setFileLength(parseHex(fcpMessage.getParameter("FileLength")));
			lastSegmentHeader.setOffset(parseHex(fcpMessage.getParameter("Offset")));
			lastSegmentHeader.setBlockCount((int) parseHex(fcpMessage.getParameter("BlockCount")));
			lastSegmentHeader.setBlockSize((int) parseHex(fcpMessage.getParameter("BlockSize")));
			lastSegmentHeader.setCheckBlockCount((int) parseHex(fcpMessage.getParameter("CheckBlockCount")));
			lastSegmentHeader.setCheckBlockSize((int) parseHex(fcpMessage.getParameter("CheckBlockSize")));
			lastSegmentHeader.setSegmentCount((int) parseHex(fcpMessage.getParameter("Segments")));
			lastSegmentHeader.setSegment((int) parseHex(fcpMessage.getParameter("SegmentNum")));
			lastSegmentHeader.setBlocksRequired((int) parseHex(fcpMessage.getParameter("BlocksRequired")));
		} else if ("BlockMap".equals(fcpMessage.getName())) {
			int dataBlockCount = lastSegmentHeader.getBlockCount();
			int checkBlockCount = lastSegmentHeader.getCheckBlockCount();
			BlockMap blockMap = new BlockMap(dataBlockCount, checkBlockCount);
			blockMaps.add(blockMap);
			for (int dataIndex = 0; dataIndex < dataBlockCount; dataIndex++) {
				blockMap.setDataBlock(dataIndex, fcpMessage.getParameter("Block." + encodeHex(dataIndex)));
			}
			for (int checkIndex = 0; checkIndex < checkBlockCount; checkIndex++) {
				blockMap.setCheckBlock(checkIndex, fcpMessage.getParameter("Check." + encodeHex(checkIndex)));
			}
			complete = (blockMaps.size() == lastSegmentHeader.getSegmentCount());
		}
	}

}
